@@ -469,6 +469,24 @@ The default policy is to use I<native> when the backend is a block device
 kvm uses I<threads> by default). Please note that if the caching policy
 is not set to I<off>, kvm may choose to fall back to using I<threads>.
 
+=item C<block_size> = I<size in bytes>
+
+Configure a specific value for the block devices your guests sees. This
+value forms the default for C<physical_block_size> and
+C<logical_block_size>.
+
+=item C<physical_block_size> = I<size in bytes>
+
+Configure the block size used by the disk controller. Defaults to
+C<block_size> if it is set. If not, no value is configured and the default
+is left to kvm.
+
+=item C<logical_block_size> = I<size in bytes>
+
+Configure the block size reported to the guest operating system. Defaults
+to C<block_size> if it is set. If not, no value is configured and the
+default is left to kvm.
+
 =item C<kvm> = I<path to kvm>
 
 The path to the kvm executable to use for starting this VM. The following
