@@ -10,6 +10,9 @@ merge virtio => undef;
 merge virtconsole;
 merge cache => undef;
 merge aio => undef;
+merge block_size => undef;
+merge logical_block_size => undef;
+merge physical_block_size => undef;
 merge serialport => 0;
 merge mem => undef;
 merge cpus => undef;
@@ -80,6 +83,22 @@ sub set_disk {
 	push @{$self->disks}, shift;
 }
 
+sub set_aio {
+	$self->aio(shift);
+}
+
+sub set_block_size {
+	$self->block_size(int(shift));
+}
+
+sub set_logical_block_size {
+	$self->logical_block_size(int(shift));
+}
+
+sub set_physical_block_size {
+	$self->physical_block_size(int(shift));
+}
+
 sub set_cache {
 	$self->cache(shift);
 }
