@@ -42,6 +42,9 @@ configvar virtio;
 configvar virtconsole => sub { $self->virtio };
 configvar cache;
 configvar aio;
+configvar block_size;
+configvar logical_block_size => sub { $self->block_size };
+configvar physical_block_size => sub { $self->block_size };
 configvar serialport;
 configvar kvm;
 configvar usb => sub { $self->tablet };
@@ -51,17 +54,20 @@ configvar cpus;
 configvar acpi;
 configvar platform;
 configvar ovmfdir;
+
 configvar chipset => sub {
 	my $platform = $self->platform;
 	return undef unless defined $platform;
 	return undef if $platform eq 'bios';
 	return 'q35';
 };
+
 configvar tablet => sub {
 	return exists $self->{tablet}
 		? $self->{tablet}
 		: $self->{vnc};
 };
+
 configvar hugepages => sub {
 	my $hugepages;
 
@@ -152,8 +158,8 @@ sub nictype {
 	return $self->virtio ? 'virtio-net' : 'e1000';
 }
 
-sub disktype {
-	return $self->virtio ? 'virtio-blk' : 'ide-hd';
+sub disk_type {
+	return $self->virtio ? 'scsi-hd' : 'ide-hd';
 }
 
 sub running {
@@ -273,6 +279,7 @@ sub devices_stanza {
 	while(@_) {
 		my $key = shift;
 		my $val = shift;
+		next unless defined $val;
 		utf8::encode($val);
 		die "value '$val' invalid because it contains quotes\n"
 			if $val =~ /"/;
@@ -288,8 +295,11 @@ sub devices_stanza {
 
 sub devices_write {
 	my $fh = shift;
+
 	my $rundir = $self->rundir;
 	my $statedir = $self->statedir;
+
+	my $id = $self->id;
 	my $name = $self->name;
 
 	$fh->write("# qemu config file\n\n")
@@ -306,9 +316,11 @@ sub devices_write {
 		if defined $mem;
 
 	my $chipset = $self->chipset;
-	$self->devices_stanza($fh, machine => undef, type => $chipset)
+	$self->devices_stanza($fh, machine => undef, type => $chipset, accel => 'kvm')
 		if defined $chipset;
 
+	$self->devices_stanza($fh, device => 'root', driver => 'pcie-root-port', slot => 0);
+
 	$self->devices_stanza($fh, chardev => 'monitor',
 		backend => 'socket',
 		server => 'on',
@@ -334,12 +346,9 @@ sub devices_write {
 		pretty => 'on',
 	);
 
-	if($self->vnc) {
-		my $id = $self->id;
-		$self->devices_stanza($fh, vnc => 'default',
-			vnc => "localhost:$id",
-		);
-	}
+	$self->devices_stanza($fh, vnc => 'default',
+		vnc => $self->vnc ? "localhost:$id" : 'none',
+	);
 
 	if(defined $self->serialport) {
 		for(my $i = 0; $i < 4; $i++) {
@@ -416,35 +425,59 @@ sub devices_write {
 	}
 
 	my $disks = $self->disks;
-	my $disktype = $self->disktype;
+	my $disk_type = $self->disk_type;
+	my $bus;
+
+	if($disk_type eq 'scsi-hd') {
+		$self->devices_stanza($fh, object => "iothread",
+			'qom-type' => 'iothread',
+		);
+		$self->devices_stanza($fh, device => "virtio-scsi",
+			driver => 'virtio-scsi-pci',
+			iothread => 'iothread',
+		);
+		$bus = "virtio-scsi.0";
+	}
+
+	my %aio_cache_default = (none => 'native', directsync => 'native');
+
 	while(my ($i, $disk) = each @$disks) {
 		die "no such file or directory: $disk\n"
 			unless -e $disk;
 
-		my %opt;
+		my $format;
 		my $cache = $self->cache;
 		my $aio = $self->aio;
+		$disk =~ s/^(\w+:)//;
+		my $backing_type = $1;
 		if(-b _) {
+			$backing_type //= 'host_device:';
 			$cache //= 'none';
-			$aio //= 'native' if $cache eq 'none';
-			$opt{format} = 'raw';
+			$aio //= $aio_cache_default{$cache};
+			$format = 'raw';
+		} else {
+			$backing_type //= 'file:';
 		}
-		$opt{cache} = $cache if defined $cache;
-		$opt{aio} = $aio if defined $aio;
 
-		$self->devices_stanza($fh, drive => "blk-$i",
-			file => $disk,
+		$self->devices_stanza($fh, drive => "drive-$i",
+			file => "$backing_type$disk",
+			format => $format,
 			if => 'none',
-			%opt,
+			discard => 'on',
+			cache => $cache,
+			aio => $aio,
 		);
 
 		my $serial = substr(md5_base64($disk), 0, 20);
 		$serial =~ tr{+/}{XY};
 
-		$self->devices_stanza($fh, device => "blk-$i",
-			driver => $disktype,
-			drive => "blk-$i",
+		$self->devices_stanza($fh, device => "hd-$i",
+			driver => $disk_type,
+			bus => $bus,
+			drive => "drive-$i",
 			serial => $serial,
+			logical_block_size => $self->logical_block_size,
+			physical_block_size => $self->physical_block_size,
 		);
 	}
 
@@ -465,14 +498,6 @@ sub devices_write {
 		);
 	}
 
-#	$self->devices_stanza($fh, drive => 'cdrom',
-#		if => 'none',
-#	);
-#	$self->devices_stanza($fh, device => 'ide0-cd1',
-#		driver => 'ide-cd',
-#		drive => 'cdrom',
-#	);
-
 	my $sections = $self->sections;
 	foreach my $section (@$sections) {
 		$self->devices_stanza($fh, @$section);
